@@ -2,7 +2,7 @@
  *    GDI Interop
  *
  * Copyright 2011 Huw Davies
- * Copyright 2012, 2014-2016 Nikolay Sivov for CodeWeavers
+ * Copyright 2012, 2014-2018 Nikolay Sivov for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,12 +38,13 @@ struct dib_data {
     int width;
 };
 
-struct rendertarget {
+struct rendertarget
+{
     IDWriteBitmapRenderTarget1 IDWriteBitmapRenderTarget1_iface;
     ID2D1SimplifiedGeometrySink ID2D1SimplifiedGeometrySink_iface;
-    LONG ref;
+    LONG refcount;
 
-    IDWriteFactory5 *factory;
+    IDWriteFactory7 *factory;
     DWRITE_TEXT_ANTIALIAS_MODE antialiasmode;
     FLOAT ppdip;
     DWRITE_MATRIX m;
@@ -52,10 +53,18 @@ struct rendertarget {
     struct dib_data dib;
 };
 
-struct gdiinterop {
+struct gdiinterop
+{
     IDWriteGdiInterop1 IDWriteGdiInterop1_iface;
+    IDWriteFontFileLoader IDWriteFontFileLoader_iface;
+    LONG refcount;
+    IDWriteFactory7 *factory;
+};
+
+struct memresource_stream {
+    IDWriteFontFileStream IDWriteFontFileStream_iface;
     LONG ref;
-    IDWriteFactory5 *factory;
+    DWORD key;
 };
 
 static inline int get_dib_stride(int width, int bpp)
@@ -111,6 +120,16 @@ static inline struct gdiinterop *impl_from_IDWriteGdiInterop1(IDWriteGdiInterop1
     return CONTAINING_RECORD(iface, struct gdiinterop, IDWriteGdiInterop1_iface);
 }
 
+static inline struct gdiinterop *impl_from_IDWriteFontFileLoader(IDWriteFontFileLoader *iface)
+{
+    return CONTAINING_RECORD(iface, struct gdiinterop, IDWriteFontFileLoader_iface);
+}
+
+static inline struct memresource_stream *impl_from_IDWriteFontFileStream(IDWriteFontFileStream *iface)
+{
+    return CONTAINING_RECORD(iface, struct memresource_stream, IDWriteFontFileStream_iface);
+}
+
 static HRESULT WINAPI rendertarget_sink_QueryInterface(ID2D1SimplifiedGeometrySink *iface, REFIID riid, void **obj)
 {
     if (IsEqualIID(riid, &IID_ID2D1SimplifiedGeometrySink) ||
@@ -232,27 +251,29 @@ static HRESULT WINAPI rendertarget_QueryInterface(IDWriteBitmapRenderTarget1 *if
 
 static ULONG WINAPI rendertarget_AddRef(IDWriteBitmapRenderTarget1 *iface)
 {
-    struct rendertarget *This = impl_from_IDWriteBitmapRenderTarget1(iface);
-    ULONG ref = InterlockedIncrement(&This->ref);
-    TRACE("(%p)->(%d)\n", This, ref);
-    return ref;
+    struct rendertarget *target = impl_from_IDWriteBitmapRenderTarget1(iface);
+    ULONG refcount = InterlockedIncrement(&target->refcount);
+
+    TRACE("%p, refcount %u.\n", iface, refcount);
+
+    return refcount;
 }
 
 static ULONG WINAPI rendertarget_Release(IDWriteBitmapRenderTarget1 *iface)
 {
-    struct rendertarget *This = impl_from_IDWriteBitmapRenderTarget1(iface);
-    ULONG ref = InterlockedDecrement(&This->ref);
+    struct rendertarget *target = impl_from_IDWriteBitmapRenderTarget1(iface);
+    ULONG refcount = InterlockedDecrement(&target->refcount);
 
-    TRACE("(%p)->(%d)\n", This, ref);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
-    if (!ref)
+    if (!refcount)
     {
-        IDWriteFactory5_Release(This->factory);
-        DeleteDC(This->hdc);
-        heap_free(This);
+        IDWriteFactory7_Release(target->factory);
+        DeleteDC(target->hdc);
+        heap_free(target);
     }
 
-    return ref;
+    return refcount;
 }
 
 static inline DWORD *get_pixel_ptr_32(struct dib_data *dib, int x, int y)
@@ -324,22 +345,22 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
     DWRITE_GLYPH_RUN const *run, IDWriteRenderingParams *params, COLORREF color,
     RECT *bbox_ret)
 {
-    struct rendertarget *This = impl_from_IDWriteBitmapRenderTarget1(iface);
+    struct rendertarget *target = impl_from_IDWriteBitmapRenderTarget1(iface);
     IDWriteGlyphRunAnalysis *analysis;
     DWRITE_RENDERING_MODE1 rendermode;
     DWRITE_GRID_FIT_MODE gridfitmode;
     DWRITE_TEXTURE_TYPE texturetype;
     DWRITE_GLYPH_RUN scaled_run;
     IDWriteFontFace3 *fontface;
-    RECT target, bounds;
+    RECT target_rect, bounds;
     HRESULT hr;
 
-    TRACE("(%p)->(%.2f %.2f %d %p %p 0x%08x %p)\n", This, originX, originY,
+    TRACE("%p, %.8e, %.8e, %d, %p, %p, 0x%08x, %p.\n", iface, originX, originY,
         measuring_mode, run, params, color, bbox_ret);
 
     SetRectEmpty(bbox_ret);
 
-    if (!This->dib.ptr)
+    if (!target->dib.ptr)
         return S_OK;
 
     if (!params)
@@ -350,18 +371,19 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
         return hr;
     }
 
-    hr = IDWriteFontFace3_GetRecommendedRenderingMode(fontface, run->fontEmSize, This->ppdip * 96.0f,
-            This->ppdip * 96.0f, NULL /* FIXME */, run->isSideways, DWRITE_OUTLINE_THRESHOLD_ALIASED, measuring_mode,
+    hr = IDWriteFontFace3_GetRecommendedRenderingMode(fontface, run->fontEmSize, target->ppdip * 96.0f,
+            target->ppdip * 96.0f, NULL /* FIXME */, run->isSideways, DWRITE_OUTLINE_THRESHOLD_ALIASED, measuring_mode,
             params, &rendermode, &gridfitmode);
     IDWriteFontFace3_Release(fontface);
     if (FAILED(hr))
         return hr;
 
-    SetRect(&target, 0, 0, This->size.cx, This->size.cy);
+    SetRect(&target_rect, 0, 0, target->size.cx, target->size.cy);
 
-    if (rendermode == DWRITE_RENDERING_MODE1_OUTLINE) {
+    if (rendermode == DWRITE_RENDERING_MODE1_OUTLINE)
+    {
         static const XFORM identity = { 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f };
-        const DWRITE_MATRIX *m = &This->m;
+        const DWRITE_MATRIX *m = &target->m;
         XFORM xform;
 
         /* target allows any transform to be set, filter it here */
@@ -380,40 +402,42 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
             xform.eDx  = m->m11 * originX + m->m21 * originY + m->dx;
             xform.eDy  = m->m12 * originX + m->m22 * originY + m->dy;
         }
-        SetWorldTransform(This->hdc, &xform);
+        SetWorldTransform(target->hdc, &xform);
 
-        BeginPath(This->hdc);
+        BeginPath(target->hdc);
 
-        hr = IDWriteFontFace_GetGlyphRunOutline(run->fontFace, run->fontEmSize * This->ppdip,
+        hr = IDWriteFontFace_GetGlyphRunOutline(run->fontFace, run->fontEmSize * target->ppdip,
             run->glyphIndices, run->glyphAdvances, run->glyphOffsets, run->glyphCount,
-            run->isSideways, run->bidiLevel & 1, &This->ID2D1SimplifiedGeometrySink_iface);
+            run->isSideways, run->bidiLevel & 1, &target->ID2D1SimplifiedGeometrySink_iface);
 
-        EndPath(This->hdc);
+        EndPath(target->hdc);
 
-        if (hr == S_OK) {
+        if (hr == S_OK)
+        {
             HBRUSH brush = CreateSolidBrush(color);
 
-            SelectObject(This->hdc, brush);
+            SelectObject(target->hdc, brush);
 
-            FillPath(This->hdc);
+            FillPath(target->hdc);
 
             /* FIXME: one way to get affected rectangle bounds is to use region fill */
             if (bbox_ret)
-                *bbox_ret = target;
+                *bbox_ret = target_rect;
 
             DeleteObject(brush);
         }
 
-        SetWorldTransform(This->hdc, &identity);
+        SetWorldTransform(target->hdc, &identity);
 
         return hr;
     }
 
     scaled_run = *run;
-    scaled_run.fontEmSize *= This->ppdip;
-    hr = IDWriteFactory5_CreateGlyphRunAnalysis(This->factory, &scaled_run, &This->m, rendermode, measuring_mode,
-            gridfitmode, This->antialiasmode, originX, originY, &analysis);
-    if (FAILED(hr)) {
+    scaled_run.fontEmSize *= target->ppdip;
+    hr = IDWriteFactory7_CreateGlyphRunAnalysis(target->factory, &scaled_run, &target->m, rendermode, measuring_mode,
+            gridfitmode, target->antialiasmode, originX, originY, &analysis);
+    if (FAILED(hr))
+    {
         WARN("failed to create analysis instance, 0x%08x\n", hr);
         return hr;
     }
@@ -431,8 +455,9 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
         texturetype = DWRITE_TEXTURE_CLEARTYPE_3x1;
     }
 
-    if (IntersectRect(&target, &target, &bounds)) {
-        UINT32 size = (target.right - target.left) * (target.bottom - target.top);
+    if (IntersectRect(&target_rect, &target_rect, &bounds))
+    {
+        UINT32 size = (target_rect.right - target_rect.left) * (target_rect.bottom - target_rect.top);
         BYTE *bitmap;
 
         color = colorref_to_pixel_888(color);
@@ -444,15 +469,15 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
             return E_OUTOFMEMORY;
         }
 
-        hr = IDWriteGlyphRunAnalysis_CreateAlphaTexture(analysis, texturetype, &target, bitmap, size);
+        hr = IDWriteGlyphRunAnalysis_CreateAlphaTexture(analysis, texturetype, &target_rect, bitmap, size);
         if (hr == S_OK) {
             /* blit to target dib */
             if (texturetype == DWRITE_TEXTURE_ALIASED_1x1)
-                blit_8(&This->dib, bitmap, &target, color);
+                blit_8(&target->dib, bitmap, &target_rect, color);
             else
-                blit_subpixel_888(&This->dib, This->size.cx, bitmap, &target, color);
+                blit_subpixel_888(&target->dib, target->size.cx, bitmap, &target_rect, color);
 
-            if (bbox_ret) *bbox_ret = target;
+            if (bbox_ret) *bbox_ret = target_rect;
         }
 
         heap_free(bitmap);
@@ -567,7 +592,7 @@ static const IDWriteBitmapRenderTarget1Vtbl rendertargetvtbl = {
     rendertarget_SetTextAntialiasMode
 };
 
-static HRESULT create_rendertarget(IDWriteFactory5 *factory, HDC hdc, UINT32 width, UINT32 height, IDWriteBitmapRenderTarget **ret)
+static HRESULT create_rendertarget(IDWriteFactory7 *factory, HDC hdc, UINT32 width, UINT32 height, IDWriteBitmapRenderTarget **ret)
 {
     struct rendertarget *target;
     HRESULT hr;
@@ -579,7 +604,7 @@ static HRESULT create_rendertarget(IDWriteFactory5 *factory, HDC hdc, UINT32 wid
 
     target->IDWriteBitmapRenderTarget1_iface.lpVtbl = &rendertargetvtbl;
     target->ID2D1SimplifiedGeometrySink_iface.lpVtbl = &rendertargetsinkvtbl;
-    target->ref = 1;
+    target->refcount = 1;
 
     target->hdc = CreateCompatibleDC(hdc);
     SetGraphicsMode(target->hdc, GM_ADVANCED);
@@ -593,7 +618,7 @@ static HRESULT create_rendertarget(IDWriteFactory5 *factory, HDC hdc, UINT32 wid
     target->ppdip = GetDeviceCaps(target->hdc, LOGPIXELSX) / 96.0f;
     target->antialiasmode = DWRITE_TEXT_ANTIALIAS_MODE_CLEARTYPE;
     target->factory = factory;
-    IDWriteFactory5_AddRef(factory);
+    IDWriteFactory7_AddRef(factory);
 
     *ret = (IDWriteBitmapRenderTarget*)&target->IDWriteBitmapRenderTarget1_iface;
 
@@ -623,25 +648,29 @@ static HRESULT WINAPI gdiinterop_QueryInterface(IDWriteGdiInterop1 *iface, REFII
 
 static ULONG WINAPI gdiinterop_AddRef(IDWriteGdiInterop1 *iface)
 {
-    struct gdiinterop *This = impl_from_IDWriteGdiInterop1(iface);
-    LONG ref = InterlockedIncrement(&This->ref);
-    TRACE("(%p)->(%d)\n", This, ref);
-    return ref;
+    struct gdiinterop *interop = impl_from_IDWriteGdiInterop1(iface);
+    LONG refcount = InterlockedIncrement(&interop->refcount);
+
+    TRACE("%p, refcount %u.\n", iface, refcount);
+
+    return refcount;
 }
 
 static ULONG WINAPI gdiinterop_Release(IDWriteGdiInterop1 *iface)
 {
-    struct gdiinterop *This = impl_from_IDWriteGdiInterop1(iface);
-    LONG ref = InterlockedDecrement(&This->ref);
+    struct gdiinterop *interop = impl_from_IDWriteGdiInterop1(iface);
+    LONG refcount = InterlockedDecrement(&interop->refcount);
 
-    TRACE("(%p)->(%d)\n", This, ref);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
-    if (!ref) {
-        factory_detach_gdiinterop(This->factory, iface);
-        heap_free(This);
+    if (!refcount)
+    {
+        IDWriteFactory7_UnregisterFontFileLoader(interop->factory, &interop->IDWriteFontFileLoader_iface);
+        factory_detach_gdiinterop(interop->factory, iface);
+        heap_free(interop);
     }
 
-    return ref;
+    return refcount;
 }
 
 static HRESULT WINAPI gdiinterop_CreateFontFromLOGFONT(IDWriteGdiInterop1 *iface,
@@ -727,12 +756,13 @@ struct font_fileinfo {
 
 /* Undocumented gdi32 exports, used to access actually selected font information */
 extern BOOL WINAPI GetFontRealizationInfo(HDC hdc, struct font_realization_info *info);
-extern BOOL WINAPI GetFontFileInfo(DWORD instance_id, DWORD unknown, struct font_fileinfo *info, DWORD size, DWORD *needed);
+extern BOOL WINAPI GetFontFileInfo(DWORD instance_id, DWORD unknown, struct font_fileinfo *info, SIZE_T size, SIZE_T *needed);
+extern BOOL WINAPI GetFontFileData(DWORD instance_id, DWORD unknown, UINT64 offset, void *buff, DWORD buff_size);
 
 static HRESULT WINAPI gdiinterop_CreateFontFaceFromHdc(IDWriteGdiInterop1 *iface,
     HDC hdc, IDWriteFontFace **fontface)
 {
-    struct gdiinterop *This = impl_from_IDWriteGdiInterop1(iface);
+    struct gdiinterop *interop = impl_from_IDWriteGdiInterop1(iface);
     struct font_realization_info info;
     struct font_fileinfo *fileinfo;
     DWRITE_FONT_FILE_TYPE filetype;
@@ -740,10 +770,10 @@ static HRESULT WINAPI gdiinterop_CreateFontFaceFromHdc(IDWriteGdiInterop1 *iface
     IDWriteFontFile *file;
     BOOL is_supported;
     UINT32 facenum;
-    DWORD needed;
+    SIZE_T needed;
     HRESULT hr;
 
-    TRACE("(%p)->(%p %p)\n", This, hdc, fontface);
+    TRACE("%p, %p, %p.\n", iface, hdc, fontface);
 
     *fontface = NULL;
 
@@ -773,8 +803,12 @@ static HRESULT WINAPI gdiinterop_CreateFontFaceFromHdc(IDWriteGdiInterop1 *iface
         return E_FAIL;
     }
 
-    hr = IDWriteFactory5_CreateFontFileReference(This->factory, fileinfo->path, &fileinfo->writetime,
-        &file);
+    if (*fileinfo->path)
+        hr = IDWriteFactory7_CreateFontFileReference(interop->factory, fileinfo->path, &fileinfo->writetime, &file);
+    else
+        hr = IDWriteFactory7_CreateCustomFontFileReference(interop->factory, &info.instance_id,
+                sizeof(info.instance_id), &interop->IDWriteFontFileLoader_iface, &file);
+
     heap_free(fileinfo);
     if (FAILED(hr))
         return hr;
@@ -784,7 +818,7 @@ static HRESULT WINAPI gdiinterop_CreateFontFaceFromHdc(IDWriteGdiInterop1 *iface
     if (SUCCEEDED(hr)) {
         if (is_supported)
             /* Simulations flags values match DWRITE_FONT_SIMULATIONS */
-            hr = IDWriteFactory5_CreateFontFace(This->factory, facetype, 1, &file, info.face_index,
+            hr = IDWriteFactory7_CreateFontFace(interop->factory, facetype, 1, &file, info.face_index,
                     info.simulations, fontface);
         else
             hr = DWRITE_E_FILEFORMAT;
@@ -805,14 +839,14 @@ static HRESULT WINAPI gdiinterop_CreateBitmapRenderTarget(IDWriteGdiInterop1 *if
 static HRESULT WINAPI gdiinterop1_CreateFontFromLOGFONT(IDWriteGdiInterop1 *iface,
     LOGFONTW const *logfont, IDWriteFontCollection *collection, IDWriteFont **font)
 {
-    struct gdiinterop *This = impl_from_IDWriteGdiInterop1(iface);
+    struct gdiinterop *interop = impl_from_IDWriteGdiInterop1(iface);
     IDWriteFontFamily *family;
     DWRITE_FONT_STYLE style;
     BOOL exists = FALSE;
     UINT32 index;
     HRESULT hr;
 
-    TRACE("(%p)->(%p %p %p)\n", This, logfont, collection, font);
+    TRACE("%p, %p, %p, %p.\n", iface, logfont, collection, font);
 
     *font = NULL;
 
@@ -821,7 +855,7 @@ static HRESULT WINAPI gdiinterop1_CreateFontFromLOGFONT(IDWriteGdiInterop1 *ifac
     if (collection)
         IDWriteFontCollection_AddRef(collection);
     else {
-        hr = IDWriteFactory5_GetSystemFontCollection(This->factory, FALSE, (IDWriteFontCollection1**)&collection, FALSE);
+        hr = IDWriteFactory5_GetSystemFontCollection((IDWriteFactory5 *)interop->factory, FALSE, (IDWriteFontCollection1 **)&collection, FALSE);
         if (FAILED(hr)) {
             ERR("failed to get system font collection: 0x%08x.\n", hr);
             return hr;
@@ -897,7 +931,176 @@ static const struct IDWriteGdiInterop1Vtbl gdiinteropvtbl = {
     gdiinterop1_GetMatchingFontsByLOGFONT
 };
 
-HRESULT create_gdiinterop(IDWriteFactory5 *factory, IDWriteGdiInterop1 **ret)
+static HRESULT WINAPI memresourcestream_QueryInterface(IDWriteFontFileStream *iface, REFIID riid, void **out)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+
+    TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), out);
+
+    if (IsEqualIID(&IID_IDWriteFontFileStream, riid) || IsEqualIID(&IID_IUnknown, riid)) {
+        *out = iface;
+        IDWriteFontFileStream_AddRef(iface);
+        return S_OK;
+    }
+
+    *out = NULL;
+    WARN("Unsupported interface %s.\n", debugstr_guid(riid));
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI memresourcestream_AddRef(IDWriteFontFileStream *iface)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+    ULONG ref = InterlockedIncrement(&This->ref);
+    TRACE("(%p)->(%d)\n", This, ref);
+    return ref;
+}
+
+static ULONG WINAPI memresourcestream_Release(IDWriteFontFileStream *iface)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+    ULONG ref = InterlockedDecrement(&This->ref);
+
+    TRACE("(%p)->(%d)\n", This, ref);
+
+    if (!ref)
+        heap_free(This);
+
+    return ref;
+}
+
+static HRESULT WINAPI memresourcestream_ReadFileFragment(IDWriteFontFileStream *iface, void const **fragment_start,
+    UINT64 offset, UINT64 fragment_size, void **fragment_context)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+    struct font_fileinfo fileinfo;
+    void *fragment;
+
+    TRACE("(%p)->(%p %s %s %p)\n", This, fragment_start, wine_dbgstr_longlong(offset),
+        wine_dbgstr_longlong(fragment_size), fragment_context);
+
+    *fragment_context = NULL;
+    *fragment_start = NULL;
+
+    if (!GetFontFileInfo(This->key, 0, &fileinfo, sizeof(fileinfo), NULL))
+        return E_INVALIDARG;
+
+    if ((offset >= fileinfo.size.QuadPart - 1) || (fragment_size > fileinfo.size.QuadPart - offset))
+        return E_INVALIDARG;
+
+    if (!(fragment = heap_alloc(fragment_size)))
+        return E_OUTOFMEMORY;
+
+    if (!GetFontFileData(This->key, 0, offset, fragment, fragment_size))
+        return E_FAIL;
+
+    *fragment_start = *fragment_context = fragment;
+    return S_OK;
+}
+
+static void WINAPI memresourcestream_ReleaseFileFragment(IDWriteFontFileStream *iface, void *fragment_context)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+
+    TRACE("(%p)->(%p)\n", This, fragment_context);
+
+    heap_free(fragment_context);
+}
+
+static HRESULT WINAPI memresourcestream_GetFileSize(IDWriteFontFileStream *iface, UINT64 *size)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+    struct font_fileinfo fileinfo;
+
+    TRACE("(%p)->(%p)\n", This, size);
+
+    if (!GetFontFileInfo(This->key, 0, &fileinfo, sizeof(fileinfo), NULL))
+        return E_INVALIDARG;
+
+    *size = fileinfo.size.QuadPart;
+
+    return S_OK;
+}
+
+static HRESULT WINAPI memresourcestream_GetLastWriteTime(IDWriteFontFileStream *iface, UINT64 *last_writetime)
+{
+    struct memresource_stream *This = impl_from_IDWriteFontFileStream(iface);
+
+    TRACE("(%p)->(%p)\n", This, last_writetime);
+
+    return E_NOTIMPL;
+}
+
+static const struct IDWriteFontFileStreamVtbl memresourcestreamvtbl = {
+    memresourcestream_QueryInterface,
+    memresourcestream_AddRef,
+    memresourcestream_Release,
+    memresourcestream_ReadFileFragment,
+    memresourcestream_ReleaseFileFragment,
+    memresourcestream_GetFileSize,
+    memresourcestream_GetLastWriteTime,
+};
+
+static HRESULT WINAPI memresourceloader_QueryInterface(IDWriteFontFileLoader *iface, REFIID riid, void **out)
+{
+    struct gdiinterop *This = impl_from_IDWriteFontFileLoader(iface);
+
+    TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), out);
+
+    if (IsEqualIID(&IID_IDWriteFontFileLoader, riid) || IsEqualIID(&IID_IUnknown, riid)) {
+        *out = iface;
+        IDWriteFontFileLoader_AddRef(iface);
+        return S_OK;
+    }
+
+    *out = NULL;
+    WARN("Unsupported interface %s.\n", debugstr_guid(riid));
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI memresourceloader_AddRef(IDWriteFontFileLoader *iface)
+{
+    return 2;
+}
+
+static ULONG WINAPI memresourceloader_Release(IDWriteFontFileLoader *iface)
+{
+    return 1;
+}
+
+static HRESULT WINAPI memresourceloader_CreateStreamFromKey(IDWriteFontFileLoader *iface, void const *key,
+        UINT32 key_size, IDWriteFontFileStream **ret)
+{
+    struct gdiinterop *This = impl_from_IDWriteFontFileLoader(iface);
+    struct memresource_stream *stream;
+
+    TRACE("(%p)->(%p %u %p)\n", This, key, key_size, ret);
+
+    *ret = NULL;
+
+    if (!key || key_size != sizeof(DWORD))
+        return E_INVALIDARG;
+
+    if (!(stream = heap_alloc(sizeof(*stream))))
+        return E_OUTOFMEMORY;
+
+    stream->IDWriteFontFileStream_iface.lpVtbl = &memresourcestreamvtbl;
+    stream->ref = 1;
+    memcpy(&stream->key, key, sizeof(stream->key));
+
+    *ret = &stream->IDWriteFontFileStream_iface;
+
+    return S_OK;
+}
+
+static const struct IDWriteFontFileLoaderVtbl memresourceloadervtbl = {
+    memresourceloader_QueryInterface,
+    memresourceloader_AddRef,
+    memresourceloader_Release,
+    memresourceloader_CreateStreamFromKey,
+};
+
+HRESULT create_gdiinterop(IDWriteFactory7 *factory, IDWriteGdiInterop1 **ret)
 {
     struct gdiinterop *interop;
 
@@ -907,8 +1110,11 @@ HRESULT create_gdiinterop(IDWriteFactory5 *factory, IDWriteGdiInterop1 **ret)
         return E_OUTOFMEMORY;
 
     interop->IDWriteGdiInterop1_iface.lpVtbl = &gdiinteropvtbl;
-    interop->ref = 1;
-    IDWriteFactory5_AddRef(interop->factory = factory);
+    interop->IDWriteFontFileLoader_iface.lpVtbl = &memresourceloadervtbl;
+    interop->refcount = 1;
+    interop->factory = factory;
+    IDWriteFactory7_AddRef(interop->factory);
+    IDWriteFactory7_RegisterFontFileLoader(factory, &interop->IDWriteFontFileLoader_iface);
 
     *ret = &interop->IDWriteGdiInterop1_iface;
     return S_OK;
