@@ -92,73 +92,6 @@ static HANDLE create_file_OF( LPCSTR path, INT mode )
 }
 
 
-/***********************************************************************
- *           FILE_SetDosError
- *
- * Set the DOS error code from errno.
- */
-void FILE_SetDosError(void)
-{
-    int save_errno = errno; /* errno gets overwritten by printf */
-
-    TRACE("errno = %d %s\n", errno, strerror(errno));
-    switch (save_errno)
-    {
-    case EAGAIN:
-        SetLastError( ERROR_SHARING_VIOLATION );
-        break;
-    case EBADF:
-        SetLastError( ERROR_INVALID_HANDLE );
-        break;
-    case ENOSPC:
-        SetLastError( ERROR_HANDLE_DISK_FULL );
-        break;
-    case EACCES:
-    case EPERM:
-    case EROFS:
-        SetLastError( ERROR_ACCESS_DENIED );
-        break;
-    case EBUSY:
-        SetLastError( ERROR_LOCK_VIOLATION );
-        break;
-    case ENOENT:
-        SetLastError( ERROR_FILE_NOT_FOUND );
-        break;
-    case EISDIR:
-        SetLastError( ERROR_CANNOT_MAKE );
-        break;
-    case ENFILE:
-    case EMFILE:
-        SetLastError( ERROR_TOO_MANY_OPEN_FILES );
-        break;
-    case EEXIST:
-        SetLastError( ERROR_FILE_EXISTS );
-        break;
-    case EINVAL:
-    case ESPIPE:
-        SetLastError( ERROR_SEEK );
-        break;
-    case ENOTEMPTY:
-        SetLastError( ERROR_DIR_NOT_EMPTY );
-        break;
-    case ENOEXEC:
-        SetLastError( ERROR_BAD_FORMAT );
-        break;
-    case ENOTDIR:
-        SetLastError( ERROR_PATH_NOT_FOUND );
-        break;
-    case EXDEV:
-        SetLastError( ERROR_NOT_SAME_DEVICE );
-        break;
-    default:
-        WARN("unknown file error: %s\n", strerror(save_errno) );
-        SetLastError( ERROR_GEN_FAILURE );
-        break;
-    }
-    errno = save_errno;
-}
-
-
 /***********************************************************************
  *           FILE_name_AtoW
  *
