@@ -24,6 +24,7 @@
 #include "shlwapi.h"
 #include "winternl.h"
 
+#include "kernelbase.h"
 #include "wine/debug.h"
 #include "wine/exception.h"
 
@@ -33,13 +34,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(string);
                       ((ch) >= 'A' && (ch) <= 'F') || \
                       ((ch) >= 'a' && (ch) <= 'f'))
 
-static WORD get_char_type(WCHAR ch)
-{
-    WORD type = 0;
-    GetStringTypeW(CT_CTYPE1, &ch, 1, &type);
-    return type;
-}
-
 static BOOL char_compare(WORD ch1, WORD ch2, DWORD flags)
 {
     char str1[3], str2[3];
@@ -219,90 +213,6 @@ DWORD WINAPI StrCmpNICW(const WCHAR *str, const WCHAR *cmp, DWORD len)
     return StrCmpNIW(str, cmp, len);
 }
 
-BOOL WINAPI IsCharBlankW(WCHAR wc)
-{
-    return !!(get_char_type(wc) & C1_BLANK);
-}
-
-BOOL WINAPI IsCharCntrlW(WCHAR wc)
-{
-    return !!(get_char_type(wc) & C1_CNTRL);
-}
-
-BOOL WINAPI IsCharDigitW(WCHAR wc)
-{
-    return !!(get_char_type(wc) & C1_DIGIT);
-}
-
-BOOL WINAPI IsCharPunctW(WCHAR wc)
-{
-    return !!(get_char_type(wc) & C1_PUNCT);
-}
-
-BOOL WINAPI IsCharSpaceA(CHAR c)
-{
-    WORD type;
-    return GetStringTypeA(GetSystemDefaultLCID(), CT_CTYPE1, &c, 1, &type) && (type & C1_SPACE);
-}
-
-BOOL WINAPI IsCharSpaceW(WCHAR wc)
-{
-    return !!(get_char_type(wc) & C1_SPACE);
-}
-
-BOOL WINAPI IsCharXDigitW(WCHAR wc)
-{
-    return !!(get_char_type(wc) & C1_XDIGIT);
-}
-
-BOOL WINAPI IsCharAlphaA(CHAR x)
-{
-    WCHAR wch;
-    MultiByteToWideChar(CP_ACP, 0, &x, 1, &wch, 1);
-    return IsCharAlphaW(wch);
-}
-
-BOOL WINAPI IsCharAlphaW(WCHAR ch)
-{
-    return !!(get_char_type(ch) & C1_ALPHA);
-}
-
-BOOL WINAPI IsCharLowerA(CHAR x)
-{
-    WCHAR wch;
-    MultiByteToWideChar(CP_ACP, 0, &x, 1, &wch, 1);
-    return IsCharLowerW(wch);
-}
-
-BOOL WINAPI IsCharLowerW(WCHAR ch)
-{
-    return !!(get_char_type(ch) & C1_LOWER);
-}
-
-BOOL WINAPI IsCharAlphaNumericA(CHAR x)
-{
-    WCHAR wch;
-    MultiByteToWideChar(CP_ACP, 0, &x, 1, &wch, 1);
-    return IsCharAlphaNumericW(wch);
-}
-
-BOOL WINAPI IsCharAlphaNumericW(WCHAR ch)
-{
-    return !!(get_char_type(ch) & (C1_ALPHA | C1_DIGIT));
-}
-
-BOOL WINAPI IsCharUpperA(CHAR x)
-{
-    WCHAR wch;
-    MultiByteToWideChar(CP_ACP, 0, &x, 1, &wch, 1);
-    return IsCharUpperW(wch);
-}
-
-BOOL WINAPI IsCharUpperW(WCHAR ch)
-{
-    return !!(get_char_type(ch) & C1_UPPER);
-}
-
 char * WINAPI StrChrA(const char *str, WORD ch)
 {
     TRACE("%s, %#x\n", wine_dbgstr_a(str), ch);
@@ -506,7 +416,7 @@ WCHAR * WINAPI StrStrNIW(const WCHAR *str, const WCHAR *search, UINT max_len)
 
     for (i = max_len; *str && (i > 0); i--, str++)
     {
-        if (!wcsnicmp(str, search, len))
+        if (!StrCmpNIW(str, search, len))
             return (WCHAR *)str;
     }
 
@@ -939,8 +849,7 @@ BOOL WINAPI StrToInt64ExA(const char *str, DWORD flags, LONGLONG *ret)
         WARN("Unknown flags %#x\n", flags);
 
     /* Skip leading space, '+', '-' */
-    while (*str == ' ' || (*str >= '\t' && *str <= '\r'))
-        str = CharNextA(str);
+    while (*str == ' ' || *str == '\t' || *str == '\n') str++;
 
     if (*str == '-')
     {
@@ -1003,8 +912,7 @@ BOOL WINAPI StrToInt64ExW(const WCHAR *str, DWORD flags, LONGLONG *ret)
         WARN("Unknown flags %#x.\n", flags);
 
     /* Skip leading space, '+', '-' */
-    while (iswspace(*str))
-        str++;
+    while (*str == ' ' || *str == '\t' || *str == '\n') str++;
 
     if (*str == '-')
     {
@@ -1014,21 +922,23 @@ BOOL WINAPI StrToInt64ExW(const WCHAR *str, DWORD flags, LONGLONG *ret)
     else if (*str == '+')
         str++;
 
-    if (flags & STIF_SUPPORT_HEX && *str == '0' && towlower(str[1]) == 'x')
+    if (flags & STIF_SUPPORT_HEX && *str == '0' && (str[1] == 'x' || str[1] == 'X'))
     {
         /* Read hex number */
         str += 2;
 
-        if (!iswxdigit(*str))
+        if (!isxdigit(*str))
             return FALSE;
 
-        while (iswxdigit(*str))
+        while (isxdigit(*str))
         {
             value *= 16;
-            if (iswdigit(*str))
+            if (*str >= '0' && *str <= '9')
                 value += (*str - '0');
+            else if (*str >= 'A' && *str <= 'Z')
+                value += 10 + (*str - 'A');
             else
-                value += 10 + (towlower(*str) - 'a');
+                value += 10 + (*str - 'a');
             str++;
         }
 
@@ -1037,10 +947,10 @@ BOOL WINAPI StrToInt64ExW(const WCHAR *str, DWORD flags, LONGLONG *ret)
     }
 
     /* Read decimal number */
-    if (!iswdigit(*str))
+    if (*str < '0' || *str > '9')
         return FALSE;
 
-    while (iswdigit(*str))
+    while (*str >= '0' && *str <= '9')
     {
         value *= 10;
         value += (*str - '0');
@@ -1084,7 +994,7 @@ int WINAPI StrToIntA(const char *str)
     if (!str)
         return 0;
 
-    if (*str == '-' || isdigit(*str))
+    if (*str == '-' || (*str >= '0' && *str <= '9'))
         StrToIntExA(str, 0, &value);
 
     return value;
@@ -1099,7 +1009,7 @@ int WINAPI StrToIntW(const WCHAR *str)
     if (!str)
         return 0;
 
-    if (*str == '-' || iswdigit(*str))
+    if (*str == '-' || (*str >= '0' && *str <= '9'))
         StrToIntExW(str, 0, &value);
     return value;
 }
@@ -1382,8 +1292,7 @@ INT WINAPI DECLSPEC_HOTPATCH LoadStringA(HINSTANCE instance, UINT resource_id, L
 
         while (id--) p += *p + 1;
 
-        if (buflen != 1)
-            RtlUnicodeToMultiByteN(buffer, buflen - 1, &retval, p + 1, *p * sizeof(WCHAR));
+        RtlUnicodeToMultiByteN(buffer, buflen - 1, &retval, p + 1, *p * sizeof(WCHAR));
     }
     buffer[retval] = 0;
     TRACE("returning %s\n", debugstr_a(buffer));
@@ -1401,11 +1310,11 @@ int WINAPI StrCmpLogicalW(const WCHAR *str, const WCHAR *comp)
     {
         if (!*comp)
             return 1;
-        else if (iswdigit(*str))
+        else if (*str >= '0' && *str <= '9')
         {
             int str_value, comp_value;
 
-            if (!iswdigit(*comp))
+            if (*comp < '0' || *comp > '9')
                 return -1;
 
             /* Compare the numbers */
@@ -1418,12 +1327,10 @@ int WINAPI StrCmpLogicalW(const WCHAR *str, const WCHAR *comp)
                 return 1;
 
             /* Skip */
-            while (iswdigit(*str))
-                str++;
-            while (iswdigit(*comp))
-                comp++;
+            while (*str >= '0' && *str <= '9') str++;
+            while (*comp >= '0' && *comp <= '9') comp++;
         }
-        else if (iswdigit(*comp))
+        else if (*comp >= '0' && *comp <= '9')
             return 1;
         else
         {
