@@ -213,8 +213,6 @@ ULONGLONG WINAPI DECLSPEC_HOTPATCH GetTickCount64(void)
 
 static HANDLE normalize_handle_if_console( HANDLE handle )
 {
-    static HANDLE wait_event;
-
     if ((handle == (HANDLE)STD_INPUT_HANDLE) ||
         (handle == (HANDLE)STD_OUTPUT_HANDLE) ||
         (handle == (HANDLE)STD_ERROR_HANDLE))
@@ -223,23 +221,7 @@ static HANDLE normalize_handle_if_console( HANDLE handle )
     /* even screen buffer console handles are waitable, and are
      * handled as a handle to the console itself
      */
-    if (is_console_handle( handle ))
-    {
-        HANDLE event = 0;
-
-        SERVER_START_REQ( get_console_wait_event )
-        {
-            req->handle = wine_server_obj_handle( console_handle_map( handle ));
-            if (!wine_server_call( req )) event = wine_server_ptr_handle( reply->event );
-        }
-        SERVER_END_REQ;
-        if (event)
-        {
-            if (InterlockedCompareExchangePointer( &wait_event, event, 0 )) NtClose( event );
-            handle = wait_event;
-        }
-    }
-    return handle;
+    return is_console_handle( handle ) ? get_console_wait_handle( handle ) : handle;
 }
 
 
