@@ -1181,7 +1181,7 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
         }
         if (needed_flags & CONTEXT_CONTROL)
         {
-            context->Esp    = (DWORD)&frame->thunk_addr;
+            context->Esp    = (DWORD)&frame->ret_addr;
             context->Ebp    = frame->ebp;
             context->Eip    = frame->thunk_addr;
             context->EFlags = 0x202;
@@ -1535,14 +1535,63 @@ static void setup_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec )
     setup_raise_exception( sigcontext, stack, rec, &context );
 }
 
+
+/***********************************************************************
+ *           call_user_apc
+ */
+void WINAPI call_user_apc( CONTEXT *context_ptr, ULONG_PTR ctx, ULONG_PTR arg1,
+                           ULONG_PTR arg2, PNTAPCFUNC func )
+{
+    CONTEXT context;
+
+    if (!context_ptr)
+    {
+        context.ContextFlags = CONTEXT_FULL;
+        NtGetContextThread( GetCurrentThread(), &context );
+        context.Eax = STATUS_USER_APC;
+        context_ptr = &context;
+    }
+    pKiUserApcDispatcher( context_ptr, ctx, arg1, arg2, func );
+}
+
+
+/***********************************************************************
+ *           call_raise_user_exception_dispatcher
+ */
+__ASM_GLOBAL_FUNC( call_raise_user_exception_dispatcher,
+                   "movl %fs:0x1f8,%eax\n\t"  /* x86_thread_data()->syscall_frame */
+                   "pushl (%eax)\n\t"         /* frame->prev_frame */
+                   "popl %fs:0x1f8\n\t"
+                   "movl 4(%eax),%edi\n\t"    /* frame->edi */
+                   "movl 8(%eax),%esi\n\t"    /* frame->esi */
+                   "movl 12(%eax),%ebx\n\t"   /* frame->ebx */
+                   "movl 16(%eax),%ebp\n\t"   /* frame->ebp */
+                   "movl 4(%esp),%edx\n\t"    /* dispatcher */
+                   "leal 24(%eax),%esp\n\t"
+                   "jmp *%edx" )
+
+
+/***********************************************************************
+ *           call_user_exception_dispatcher
+ */
 __ASM_GLOBAL_FUNC( call_user_exception_dispatcher,
-                   "add $4,%esp\n\t"
-                   "movl (%esp),%eax\n\t" /* rec */
-                   "cmpl $0x80000003,(%eax)\n\t" /* ExceptionCode */
+                   "movl 4(%esp),%edx\n\t"        /* rec */
+                   "movl 8(%esp),%ecx\n\t"        /* context */
+                   "cmpl $0x80000003,(%edx)\n\t"  /* rec->ExceptionCode */
                    "jne 1f\n\t"
-                   "movl 4(%esp),%eax\n\t" /* context */
-                   "decl 0xb8(%eax)\n\t" /* Eip */
-                   "1:\tjmp *8(%esp)")
+                   "decl 0xb8(%ecx)\n"            /* context->Eip */
+                   "1:\tmovl %fs:0x1f8,%eax\n\t"  /* x86_thread_data()->syscall_frame */
+                   "pushl (%eax)\n\t"             /* frame->prev_frame */
+                   "popl %fs:0x1f8\n\t"
+                   "movl 4(%eax),%edi\n\t"        /* frame->edi */
+                   "movl 8(%eax),%esi\n\t"        /* frame->esi */
+                   "movl 12(%eax),%ebx\n\t"       /* frame->ebx */
+                   "movl 16(%eax),%ebp\n\t"       /* frame->ebp */
+                   "movl %edx,16(%eax)\n\t"
+                   "movl %ecx,20(%eax)\n\t"
+                   "movl 12(%esp),%edx\n\t"       /* dispatcher */
+                   "leal 16(%eax),%esp\n\t"
+                   "jmp *%edx" )
 
 /**********************************************************************
  *		get_fpu_code
