@@ -187,7 +187,6 @@ struct oletls
 
 
 /* Global Interface Table Functions */
-extern IGlobalInterfaceTable *get_std_git(void) DECLSPEC_HIDDEN;
 extern void release_std_git(void) DECLSPEC_HIDDEN;
 extern HRESULT StdGlobalInterfaceTable_GetFactory(LPVOID *ppv) DECLSPEC_HIDDEN;
 
@@ -319,6 +318,13 @@ extern HRESULT HandlerCF_Create(REFCLSID rclsid, REFIID riid, LPVOID *ppv) DECLS
 extern HRESULT WINAPI GlobalOptions_CreateInstance(IClassFactory *iface, IUnknown *pUnk,
                                                    REFIID riid, void **ppv) DECLSPEC_HIDDEN;
 extern IClassFactory GlobalOptionsCF DECLSPEC_HIDDEN;
+extern HRESULT WINAPI GlobalInterfaceTable_CreateInstance(IClassFactory *iface, IUnknown *outer, REFIID riid,
+        void **obj) DECLSPEC_HIDDEN;
+extern IClassFactory GlobalInterfaceTableCF DECLSPEC_HIDDEN;
+extern HRESULT WINAPI ManualResetEvent_CreateInstance(IClassFactory *iface, IUnknown *outer, REFIID riid,
+        void **obj) DECLSPEC_HIDDEN;
+extern IClassFactory ManualResetEventCF DECLSPEC_HIDDEN;
+extern HRESULT WINAPI Ole32DllGetClassObject(REFCLSID clsid, REFIID riid, void **obj) DECLSPEC_HIDDEN;
 
 /* Exported non-interface Data Advise Holder functions */
 HRESULT DataAdviseHolder_OnConnect(IDataAdviseHolder *iface, IDataObject *pDelegate) DECLSPEC_HIDDEN;
