@@ -1689,26 +1689,6 @@ HRESULT WINAPI CoGetStandardMarshal(REFIID riid, IUnknown *pUnk,
     return StdMarshalImpl_Construct(&IID_IMarshal, dwDestContext, pvDestContext, (void**)ppMarshal);
 }
 
-/***********************************************************************
- *		get_marshaler	[internal]
- *
- * Retrieves an IMarshal interface for an object.
- */
-static HRESULT get_marshaler(REFIID riid, IUnknown *pUnk, DWORD dwDestContext,
-                             void *pvDestContext, DWORD mshlFlags,
-                             LPMARSHAL *pMarshal)
-{
-    HRESULT hr;
-
-    if (!pUnk)
-        return E_POINTER;
-    hr = IUnknown_QueryInterface(pUnk, &IID_IMarshal, (LPVOID*)pMarshal);
-    if (hr != S_OK)
-        hr = CoGetStandardMarshal(riid, pUnk, dwDestContext, pvDestContext,
-                                  mshlFlags, pMarshal);
-    return hr;
-}
-
 /***********************************************************************
  *		get_unmarshaler_from_stream	[internal]
  *
@@ -1777,187 +1757,6 @@ static HRESULT get_unmarshaler_from_stream(IStream *stream, IMarshal **marshal,
     return hr;
 }
 
-/***********************************************************************
- *		CoGetMarshalSizeMax	[OLE32.@]
- *
- * Gets the maximum amount of data that will be needed by a marshal.
- *
- * PARAMS
- *  pulSize       [O] Address where maximum marshal size will be stored.
- *  riid          [I] Identifier of the interface to marshal.
- *  pUnk          [I] Pointer to the object to marshal.
- *  dwDestContext [I] Destination. Used to enable or disable optimizations.
- *  pvDestContext [I] Reserved. Must be NULL.
- *  mshlFlags     [I] Flags that affect the marshaling. See CoMarshalInterface().
- *
- * RETURNS
- *  Success: S_OK.
- *  Failure: HRESULT code.
- *
- * SEE ALSO
- *  CoMarshalInterface().
- */
-HRESULT WINAPI CoGetMarshalSizeMax(ULONG *pulSize, REFIID riid, IUnknown *pUnk,
-                                   DWORD dwDestContext, void *pvDestContext,
-                                   DWORD mshlFlags)
-{
-    HRESULT hr;
-    LPMARSHAL pMarshal;
-    BOOL std_marshal = FALSE;
-
-    if(!pUnk)
-        return E_POINTER;
-
-    hr = IUnknown_QueryInterface(pUnk, &IID_IMarshal, (void**)&pMarshal);
-    if (hr != S_OK)
-    {
-        std_marshal = TRUE;
-        hr = CoGetStandardMarshal(riid, pUnk, dwDestContext, pvDestContext,
-                                  mshlFlags, &pMarshal);
-    }
-    if (hr != S_OK)
-        return hr;
-
-    hr = IMarshal_GetMarshalSizeMax(pMarshal, riid, pUnk, dwDestContext,
-                                    pvDestContext, mshlFlags, pulSize);
-    if (!std_marshal)
-        /* add on the size of the whole OBJREF structure like native does */
-        *pulSize += sizeof(OBJREF);
-
-    IMarshal_Release(pMarshal);
-    return hr;
-}
-
-
-static void dump_MSHLFLAGS(MSHLFLAGS flags)
-{
-    if (flags & MSHLFLAGS_TABLESTRONG)
-        TRACE(" MSHLFLAGS_TABLESTRONG");
-    if (flags & MSHLFLAGS_TABLEWEAK)
-        TRACE(" MSHLFLAGS_TABLEWEAK");
-    if (!(flags & (MSHLFLAGS_TABLESTRONG|MSHLFLAGS_TABLEWEAK)))
-        TRACE(" MSHLFLAGS_NORMAL");
-    if (flags & MSHLFLAGS_NOPING)
-        TRACE(" MSHLFLAGS_NOPING");
-}
-
-/***********************************************************************
- *		CoMarshalInterface	[OLE32.@]
- *
- * Marshals an interface into a stream so that the object can then be
- * unmarshaled from another COM apartment and used remotely.
- *
- * PARAMS
- *  pStream       [I] Stream the object will be marshaled into.
- *  riid          [I] Identifier of the interface to marshal.
- *  pUnk          [I] Pointer to the object to marshal.
- *  dwDestContext [I] Destination. Used to enable or disable optimizations.
- *  pvDestContext [I] Reserved. Must be NULL.
- *  mshlFlags     [I] Flags that affect the marshaling. See notes.
- *
- * RETURNS
- *  Success: S_OK.
- *  Failure: HRESULT code.
- *
- * NOTES
- *
- * The mshlFlags parameter can take one or more of the following flags:
- *| MSHLFLAGS_NORMAL - Unmarshal once, releases stub on last proxy release.
- *| MSHLFLAGS_TABLESTRONG - Unmarshal many, release when CoReleaseMarshalData() called.
- *| MSHLFLAGS_TABLEWEAK - Unmarshal many, releases stub on last proxy release.
- *| MSHLFLAGS_NOPING - No automatic garbage collection (and so reduces network traffic).
- *
- * If a marshaled object is not unmarshaled, then CoReleaseMarshalData() must
- * be called in order to release the resources used in the marshaling.
- *
- * SEE ALSO
- *  CoUnmarshalInterface(), CoReleaseMarshalData().
- */
-HRESULT WINAPI CoMarshalInterface(IStream *pStream, REFIID riid, IUnknown *pUnk,
-                                  DWORD dwDestContext, void *pvDestContext,
-                                  DWORD mshlFlags)
-{
-    HRESULT	hr;
-    CLSID marshaler_clsid;
-    LPMARSHAL pMarshal;
-
-    TRACE("(%p, %s, %p, %x, %p, ", pStream, debugstr_guid(riid), pUnk,
-        dwDestContext, pvDestContext);
-    dump_MSHLFLAGS(mshlFlags);
-    TRACE(")\n");
-
-    if (!pUnk || !pStream)
-        return E_INVALIDARG;
-
-    /* get the marshaler for the specified interface */
-    hr = get_marshaler(riid, pUnk, dwDestContext, pvDestContext, mshlFlags, &pMarshal);
-    if (hr != S_OK)
-    {
-        ERR("Failed to get marshaller, 0x%08x\n", hr);
-        return hr;
-    }
-
-    hr = IMarshal_GetUnmarshalClass(pMarshal, riid, pUnk, dwDestContext,
-                                    pvDestContext, mshlFlags, &marshaler_clsid);
-    if (hr != S_OK)
-    {
-        ERR("IMarshal::GetUnmarshalClass failed, 0x%08x\n", hr);
-        goto cleanup;
-    }
-
-    /* FIXME: implement handler marshaling too */
-    if (IsEqualCLSID(&marshaler_clsid, &CLSID_StdMarshal))
-    {
-        TRACE("Using standard marshaling\n");
-    }
-    else
-    {
-        OBJREF objref;
-
-        TRACE("Using custom marshaling\n");
-        objref.signature = OBJREF_SIGNATURE;
-        objref.iid = *riid;
-        objref.flags = OBJREF_CUSTOM;
-        objref.u_objref.u_custom.clsid = marshaler_clsid;
-        objref.u_objref.u_custom.cbExtension = 0;
-        objref.u_objref.u_custom.size = 0;
-        hr = IMarshal_GetMarshalSizeMax(pMarshal, riid, pUnk, dwDestContext,
-                                        pvDestContext, mshlFlags,
-                                        &objref.u_objref.u_custom.size);
-        if (hr != S_OK)
-        {
-            ERR("Failed to get max size of marshal data, error 0x%08x\n", hr);
-            goto cleanup;
-        }
-        /* write constant sized common header and OR_CUSTOM data into stream */
-        hr = IStream_Write(pStream, &objref,
-                          FIELD_OFFSET(OBJREF, u_objref.u_custom.pData), NULL);
-        if (hr != S_OK)
-        {
-            ERR("Failed to write OR_CUSTOM header to stream with 0x%08x\n", hr);
-            goto cleanup;
-        }
-    }
-
-    TRACE("Calling IMarshal::MarshalInterface\n");
-    /* call helper object to do the actual marshaling */
-    hr = IMarshal_MarshalInterface(pMarshal, pStream, riid, pUnk, dwDestContext,
-                                   pvDestContext, mshlFlags);
-
-    if (hr != S_OK)
-    {
-        ERR("Failed to marshal the interface %s, %x\n", debugstr_guid(riid), hr);
-        goto cleanup;
-    }
-
-cleanup:
-    IMarshal_Release(pMarshal);
-
-    TRACE("completed with hr 0x%08x\n", hr);
-    
-    return hr;
-}
-
 /***********************************************************************
  *		CoUnmarshalInterface	[OLE32.@]
  *
@@ -2079,81 +1878,6 @@ HRESULT WINAPI CoReleaseMarshalData(IStream *pStream)
     return hr;
 }
 
-
-/***********************************************************************
- *		CoMarshalInterThreadInterfaceInStream	[OLE32.@]
- *
- * Marshal an interface across threads in the same process.
- *
- * PARAMS
- *  riid  [I] Identifier of the interface to be marshalled.
- *  pUnk  [I] Pointer to IUnknown-derived interface that will be marshalled.
- *  ppStm [O] Pointer to IStream object that is created and then used to store the marshalled interface.
- *
- * RETURNS
- *  Success: S_OK
- *  Failure: E_OUTOFMEMORY and other COM error codes
- *
- * SEE ALSO
- *   CoMarshalInterface(), CoUnmarshalInterface() and CoGetInterfaceAndReleaseStream()
- */
-HRESULT WINAPI CoMarshalInterThreadInterfaceInStream(
-    REFIID riid, LPUNKNOWN pUnk, LPSTREAM * ppStm)
-{
-    ULARGE_INTEGER	xpos;
-    LARGE_INTEGER		seekto;
-    HRESULT		hres;
-
-    TRACE("(%s, %p, %p)\n",debugstr_guid(riid), pUnk, ppStm);
-
-    hres = CreateStreamOnHGlobal(NULL, TRUE, ppStm);
-    if (FAILED(hres)) return hres;
-    hres = CoMarshalInterface(*ppStm, riid, pUnk, MSHCTX_INPROC, NULL, MSHLFLAGS_NORMAL);
-
-    if (SUCCEEDED(hres))
-    {
-        memset(&seekto, 0, sizeof(seekto));
-        IStream_Seek(*ppStm, seekto, STREAM_SEEK_SET, &xpos);
-    }
-    else
-    {
-        IStream_Release(*ppStm);
-        *ppStm = NULL;
-    }
-
-    return hres;
-}
-
-/***********************************************************************
- *		CoGetInterfaceAndReleaseStream	[OLE32.@]
- *
- * Unmarshalls an interface from a stream and then releases the stream.
- *
- * PARAMS
- *  pStm [I] Stream that contains the marshalled interface.
- *  riid [I] Interface identifier of the object to unmarshall.
- *  ppv  [O] Address of pointer where the requested interface object will be stored.
- *
- * RETURNS
- *  Success: S_OK
- *  Failure: A COM error code
- *
- * SEE ALSO
- *  CoMarshalInterThreadInterfaceInStream() and CoUnmarshalInterface()
- */
-HRESULT WINAPI CoGetInterfaceAndReleaseStream(LPSTREAM pStm, REFIID riid,
-                                              LPVOID *ppv)
-{
-    HRESULT hres;
-
-    TRACE("(%p, %s, %p)\n", pStm, debugstr_guid(riid), ppv);
-
-    if(!pStm) return E_INVALIDARG;
-    hres = CoUnmarshalInterface(pStm, riid, ppv);
-    IStream_Release(pStm);
-    return hres;
-}
-
 static HRESULT WINAPI StdMarshalCF_QueryInterface(LPCLASSFACTORY iface,
                                                   REFIID riid, LPVOID *ppv)
 {
@@ -2207,45 +1931,3 @@ HRESULT MARSHAL_GetStandardMarshalCF(LPVOID *ppv)
     *ppv = &StdMarshalCF;
     return S_OK;
 }
-
-/***********************************************************************
- *		CoMarshalHresult	[OLE32.@]
- *
- * Marshals an HRESULT value into a stream.
- *
- * PARAMS
- *  pStm    [I] Stream that hresult will be marshalled into.
- *  hresult [I] HRESULT to be marshalled.
- *
- * RETURNS
- *  Success: S_OK
- *  Failure: A COM error code
- *
- * SEE ALSO
- *  CoUnmarshalHresult().
- */
-HRESULT WINAPI CoMarshalHresult(LPSTREAM pStm, HRESULT hresult)
-{
-    return IStream_Write(pStm, &hresult, sizeof(hresult), NULL);
-}
-
-/***********************************************************************
- *		CoUnmarshalHresult	[OLE32.@]
- *
- * Unmarshals an HRESULT value from a stream.
- *
- * PARAMS
- *  pStm     [I] Stream that hresult will be unmarshalled from.
- *  phresult [I] Pointer to HRESULT where the value will be unmarshalled to.
- *
- * RETURNS
- *  Success: S_OK
- *  Failure: A COM error code
- *
- * SEE ALSO
- *  CoMarshalHresult().
- */
-HRESULT WINAPI CoUnmarshalHresult(LPSTREAM pStm, HRESULT * phresult)
-{
-    return IStream_Read(pStm, phresult, sizeof(*phresult), NULL);
-}
