@@ -70,6 +70,7 @@ static const char * const FlagNames[] =
     "ordinal",     /* FLAG_ORDINAL */
     "thiscall",    /* FLAG_THISCALL */
     "fastcall",    /* FLAG_FASTCALL */
+    "syscall",     /* FLAG_SYSCALL */
     "import",      /* FLAG_IMPORT */
     NULL
 };
@@ -320,6 +321,14 @@ static int parse_spec_arguments( ORDDEF *odp, DLLSPEC *spec, int optional )
             return 0;
         }
     }
+    if (odp->flags & FLAG_SYSCALL)
+    {
+        if (odp->type != TYPE_STDCALL)
+        {
+            error( "A syscall function must use the stdcall convention\n" );
+            return 0;
+        }
+    }
     return 1;
 }
 
@@ -827,6 +836,7 @@ void add_16bit_exports( DLLSPEC *spec32, DLLSPEC *spec16 )
 
     spec32->file_name = xstrdup( spec16->file_name );
     spec32->characteristics = IMAGE_FILE_DLL;
+    spec32->init_func = xstrdup( "DllMain" );
 
     /* add an export for the NE module */
 
